#
# ParseForm.rb
#
# Copyright 2005-2008 Tero Hasu
#

module ChaosOrder
  class OrderFormParser
    def initialize
      @items = []
    end

    # Returns a list of something like
    #
    #   ["48",
    #    "BACKLIST STAR WARS RELISTS",
    #    "MAR040139",
    #    "STAR WARS DROIDS THE KALARBA ADVENTURES TP (STAR01093) (C: 3)",
    #    "17.95"]
    #
    # where the publisher entry may be nil.
    attr_reader :items



    # xxx likely we will no longer be needing these regexps

    def self.join(*list)
      Regexp.new(list.join(''))
    end

    def self.group(pat)
      '(' + pat + ')'
    end

    def self.any(pat)
      pat + '*'
    end

    def self.some(pat)
      pat + '+'
    end

    WS_PAT = '[ \t\f\v\r]'
    SEP_PAT = '\s*\t\s*'
    STRING_PAT = '\S(?:[^\t\n]*\S)?'

    CODE_PAT = '(?:JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC)\d{6}'

    PAGE_RE = join('PAGE', some(WS_PAT), group(STRING_PAT))
    ITEM_RE = join(SEP_PAT, group(STRING_PAT), SEP_PAT,
                   group(STRING_PAT), SEP_PAT, group(STRING_PAT))
    PUBL_RE = join(STRING_PAT)
    NEWL_RE = /\n/

    #p ITEM_RE.source



    def parse(data)
      require 'csv'
      require 'stringio'
      input = StringIO.new data

      header = CSV.parse_line(input.readline || (raise "no header"))
      ix_list = %w{PAGE PUBLISHER ItemCode FULL_TITLE SRP}.map {|x| header.index(x) || "no #{x} column in data"}

      input.each_line do |line|
        next unless line =~ /\S/
        rec = CSV.parse_line(line)
        @items.push(ix_list.map {|x| rec[x]})
      end
    end
  end
end # module ChaosOrder

if $0 == __FILE__
  file = ARGV.shift || '../doc/sample-data/dia08_05.asc'
  data = File.read(file)
  parser = ChaosOrder::OrderFormParser.new
  parser.parse(data)
  for item in parser.items
    p item
  end
end
